#!/tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl
# index.cgi
# Template for PureMessage Webmin Modules

use HTML::Template;
use Data::Dumper;
use Net::SMTP;
use PureMessage::Version qw($VERSION $PREFIX);
use PureMessage::Manager::Widget::ToolBar;
use PureMessage::Manager::Request qw($REQUEST %in %text);
use PureMessage::Manager::Response::Support::SendRequest;
use PureMessage::Manager::Lang;
use PureMessage::Manager::Config qw(%config);
use PureMessage::Manager::Utils qw( html_escape);
use PureMessage::_sys;
use PureMessage::Support::Request;
use PureMessage::Support::Data;


use strict;

my $pms = new PureMessage::Support::Request;

my  %vars;
our %in;
our %text;
our %config;
our @errors;

my $RESPONSE = $REQUEST->get_response( 'PureMessage::Manager::Response::Support::SendRequest',
#                                       'stderr_to_stdout' =>1
                                     );

$RESPONSE->add_help(text => "Help", tab => "support", page => "mgr_support_req" );

eval {
 
  # get out new variables for merging with reporting
  $vars{'SEND_SUPPORT'} = 1;
  $RESPONSE->title("Send Support Request");
  $vars{HEADER} = $RESPONSE->ThemeSectionHeader(title=>text('section_title'))->output; 

  my $tb = new PureMessage::Manager::Widget::ToolBar( name    => 'toolbar',
                                                      buttons => [ [{ type => 'image', text=>'Send', 
                                                                    name=>'Send',icon=>'send.gif'
                                                                }] ],
                                                      table    => [[2,2],[undef,undef]],
                                                    );
  $vars{'FOOTER'} = $RESPONSE->ThemeSectionFooter(toolbar=>$tb)->output();



# for components list 

my @components = map { { NAME => $_ } } $pms->components();
my @requestType = map { { NAME => $_ } } $pms->types();

my $first_view = not defined $in{component}; # component is always defined when the form is submitted
                                         # so it's a good indicator if the form
                                         # has been submitted or not

$vars{SUPPORT_EMAIL_ADDR} = $in{support_email_addr} || $pms->{to};
$vars{COMPANY} = $in{company} || $pms->{company};

# import out cgi params into a hash for output back to the user
foreach my $var ( qw( email company component type name problem subject) ) {
  $vars{uc $var} = $in{$var};
}

if ($vars{'COMPONENT'}) {
  foreach my $component (@components) {
    $component->{SELECTED} = 1 if $component->{NAME} eq $vars{'COMPONENT'};
  }
}

$vars{'TYPE'} = $requestType[0] unless $vars{'TYPE'};

if ($vars{'TYPE'}) {
  foreach my $type (@requestType) {
    $type->{SELECTED} = 1 if $type->{NAME} eq $vars{'TYPE'};
  }
}


# import our dumb checkboxes which send On but expect a checked on return
#
my %available = $pms->attachment_groups();
my @groups = keys %available;


my %checked = ();
foreach my $var ( @groups ) {
  $checked{$var} = 'checked' if not $first_view and $in{"attach_$var"} =~ /on|checked/i;
  $checked{$var} = 'checked' if $first_view;
}

my @attachments;
foreach my $attach (@groups) {
  my %info;
  $info{DESCRIPTION} = $available{$attach}->{description};
  $info{NAME} = $attach;
  $info{CHECKED} = $checked{$attach} || '';
  push(@attachments,\%info);
}

# default the subject if it isn't set

$vars{SUBJECT} = text('default_email_subject') unless $vars{SUBJECT};

if ($in{'Send'}) { # means it was clicked

  # then this is the second time through here so validate the data that was
  # passed to us and if ok then send the email.
  if (not (inputValidationFailure()) ) {

    sendMail();

  } else {
  
    # display the form but flag an error
    putForm();
    
  }
  
} elsif ($in{'Send'} =~ /cancel/i or $in{'SendOk'} =~ /cancel/i) {
  %vars = (); # clear the vars hash
  putForm();

} elsif ($in{view}) {
  # then the user just want's to view the data we would send
  putView($in{view});

} else {
  # first time we've seen this page, just output the form
  putForm();
}


exit;


sub putForm {
  
  my $SET_DEFAULT_FOCUS = q|document.forms[0].elements[0].focus()|;
  $RESPONSE->{body_attrib}->{onload} = $SET_DEFAULT_FOCUS; 
  $vars{COMPONENT_LOOP} = \@components;
  $vars{TYPE_LOOP} = \@requestType;
  $vars{ATTACH_LOOP} = \@attachments;
  $RESPONSE->add_template( file=>'tmpl/index.html', data=>\%vars, die_on_bad_params => 0);
  $RESPONSE->send();


}


sub putView {
  my $what = shift;

  my $emailText = $pms->get_attachment_for_display($what);
  $vars{DATA} = html_escape($emailText);
  $vars{NAME} = $available{$what}->{description};
  $RESPONSE->{no_nav} =1; # suppress navigation eye candy
  $RESPONSE->{page_width} = 300; # make the page only 300 wide
  $RESPONSE->add_template( file => 'tmpl/view.html', data => \%vars);
  $RESPONSE->{no_wrap} =1;
  $RESPONSE->send();

}

sub sendMail {
    
  if (!$RESPONSE->error() ) { # then it's ok to send
    my $subject_extra = " | $vars{NAME} \@ $vars{COMPANY} - $in{type}";
    my $subject = $vars{'SUBJECT'}; # or maybe it's not there.. no it's there
    $pms->{subject} = $subject;
    $pms->{company} = $vars{COMPANY};
    $pms->{type} = $in{type};
    $pms->{name} = $vars{NAME};
    $pms->{email} = $vars{EMAIL};
    $pms->{component} = $in{component};
    $pms->{description} = $in{problem};
    $pms->{to} = $in{support_email_addr};
    $pms->clear_attachments();

    foreach my $value (keys %in) {
      if ($value =~ /^attach_(.+)$/) {
        if (exists $available{$1}) {
          $pms->add_attachment($1);
        }
      }
    }
    eval {
        $pms->send();
    };
    if ($@) {
      print STDERR $@;
      $RESPONSE->raise_error("err_unable_to_send_email"); 
      # something bad happened
      $vars{ERROR_FLAG} =1;
    } 
        
  } 


  # do our errors, we have to delay calling the error_dlg until this point
  # because it will clear the @errors var and it will look like nothing happened
  $vars{ERROR_FLAG} = $RESPONSE->error() ? 1 : 0;
  
  $RESPONSE->error_dlg() if $RESPONSE->error();

  my $SUPPORT_EMAIL_ADDRESS = $pms->{to};
my $TEXT = <<EOF;
  Email was sent to $SUPPORT_EMAIL_ADDRESS. Support requests are attended
  and resolved according to their assigned technical severity. Thank you
  for using PureMessage.
<BR> <BR>
  <div style="text-align:left; width=70%; margin-left: 20%">
  Click <a href=index.cgi>here</a> to send another support request. <BR>
  Click <a href=/>here</a> to go to the "Dashboard" page.
  </div>
EOF

  if (not $vars{ERROR_FLAG}) {
    $RESPONSE->dialog( style=> 'dlg_info', msg => $TEXT, title => "Mail Sent");
  }

  $RESPONSE->send();
}

sub inputValidationFailure {
 
 $RESPONSE->raise_error("err_component_validation") unless $in{'component'} =~ /./;
 $RESPONSE->raise_error("err_email_validation") unless $in{'email'} =~ /.+\@.+/;
 $RESPONSE->raise_error("err_name_validation") unless $in{'name'} =~ /./;
 $RESPONSE->raise_error("err_type_validation") unless $in{'type'} =~ /./;
 $RESPONSE->raise_error("err_problem_validation") unless $in{'problem'} =~ /./;
 $RESPONSE->raise_error("err_company_validation") unless $in{'company'} =~ /./;
 return $RESPONSE->error();
}

};

if ($@) {
        $RESPONSE->raise_error($@);
        $RESPONSE->send;
}


