package PureMessage::Support::Config;
use strict;
use PureMessage::Config qw(expand_filename);
use base qw(PureMessage::Config::Validator Exporter);

our @EXPORT_OK = qw( %config %support_config);
our %config;
our $object;

our $support_config; # alias that doesn't clash with other %configs
*support_config =\%config;

my $CONFIG_LOCATION = "etc/support.conf";

sub refresh() {
    require PureMessage::Config::Validator;

    $object = ActiveState::Config->new(strict => 1, allow_includes => 1);
    PureMessage::Config::Validator->define_core_validators($object);
    PureMessage::Support::Config->validate($object);
    $object->parse_file(expand_filename($CONFIG_LOCATION));
    %config = $object->as_hash;
    foreach (qw( log_dir)) {
	$config{$_} = $PureMessage::Config::config{$_};
    }
}

refresh();

=item edit()

 Will open this config as a writeable  ActiveState::Config object
 and return it. This can die when configuration file has invalid
 syntax.

=cut

sub edit {
    my $p = ActiveState::Config->new(strict => 1, allow_includes => 1);
    PureMessage::Config::Validator->define_core_validators($p);
    PureMessage::Support::Config->validate($p);
    $p->parse_file(expand_filename($CONFIG_LOCATION),writable => 1);
    return $p;
}

sub validate {
    my ($pkg, $p) = @_;
    $p->typedecl('max_log_lines','#int');
    $p->default('max_log_lines','5000');
    $p->typedecl('max_log_lines_display','#int');
    $p->default('max_log_lines_display','200');
    $p->typedecl('component','#str');
    $p->default('component',['PureMessage','other']);
    $p->typedecl('group#*','#any');
    
    $p->typedecl('default_subject','#str');
    $p->default('default_subject','PMX Issue');
    
    $pkg->include_sub_validators($p);
}

1;
