package PureMessage::Support::Data::Base;
use strict;
use PureMessage::Support::Config qw(%support_config);
use PureMessage::Version qw($PREFIX);
use Compress::Zlib;
use File::ReadBackwards;
use Exporter;
use base qw(Exporter);
use vars qw(@EXPORT_OK);
@EXPORT_OK = qw( get_log get_program_output get_file add_group add get_resource_data get_log_compressed);

sub add_group {
    my ($group,%config) = @_;
    $PureMessage::Support::Data::SUPPORT{$group}->{$_} = $config{$_} foreach keys %config;
}

sub add {
    my ($group,%a) = @_;
    $PureMessage::Support::Data::SUPPORT{$group}->{item}->{$_} = $a{$_} foreach keys %a;
}

# returns the contents of the requested log file, if the passed $log_name isn't
# absolute then it is assumed that it is relative to the $PREFIX location.

sub _tail {
    my ($count, $file) = @_;
    my $bw = File::ReadBackwards->new( $file ) or die "can't read $file: $!\n" ;
    my $lines;
    while( defined( my $line = $bw->readline ) ) {
        $lines = $line . $lines;
        last if --$count <= 0;
    }
    return $lines;
}

sub get_log {
    my ($log_name, $max_lines) = @_;
    my $error;
    return unless $log_name and $max_lines;
    $log_name = "$PREFIX/$log_name" unless $log_name =~ m(^/);
    return _tail($max_lines, $log_name);
}

sub get_log_compressed {
    my ($log_name, $max_lines) = @_;
    die "max_lines must be > 0" unless ($max_lines && $max_lines > 0);
    my $error;
    return unless $log_name and $max_lines;
    $log_name = "$PREFIX/$log_name" unless $log_name =~ m(^/);
    my $data = _tail($max_lines, $log_name);
    return Compress::Zlib::memGzip($data);
}

sub get_file {
    my $file = shift;
    my $password_cleaning_sub = shift || sub { return shift };
    $file = "$PREFIX/$file" unless $file =~ m(^/);
    my $text;
    return unless $file;
    return "Can't locate file: '$file'" unless -e $file;
    return "Unable to remove sensitive information from file. Not attaching."
        if $password_cleaning_sub and ref($password_cleaning_sub) ne 'CODE';
    open FV, $file || return "Can't read from file '$file'";
    while (<FV>) {
	$text .= &$password_cleaning_sub($_);
    }
    $text;
}

sub get_resource_data {
  my $name = shift;
  my $password_cleaning_sub = shift || sub { return shift };
  my $out = '';
  return "Unable to remove sensitive information from file. Not attaching."
        if $password_cleaning_sub and ref($password_cleaning_sub) ne 'CODE';
  eval {
      require PureMessage::Resources::Manager;
      my $m  = PureMessage::Resources::Manager->new();
      my $r = $m->find($name);
      die "Unable to find '$name'\n" unless $r;
      my @text = $r->to_text();
      if (scalar(@text)) {
          foreach my $l (@text) {
            $out .= &$password_cleaning_sub($l) . "\n";
          }
      } else {
        $out .= "A resource of type " . $r->type . " cannot be displayed\n\n";
      }
  };
  
  if ($@) {
    return "There was an error including the contents of '$name': $@\n";  
  } 
  return $out;
}

sub get_program_output {
    my $program = shift;
    $ENV{PREFIX} = $PREFIX;
    return unless $program;
    my $text = `$program`;
    return $? if $?;
    $text;
}

1;
