package PureMessage::Support::Data::support;
use strict;
use PureMessage::Support::Data::Base qw(add add_group get_log get_file get_program_output
                                        get_resource_data get_log_compressed);
use PureMessage::Support::Config qw( %support_config);
use PureMessage::Version qw($PREFIX);
use Compress::Zlib;

# initialize the defaults from our config file
sub _expand_files {
    my $ref = shift;
    my @files;
    my @raw_files;
    if (ref($ref) eq 'ARRAY') {
	push(@raw_files,@$ref);
    }
    else {
	push(@raw_files,$ref);
    }
    # expand any globs we see
    foreach my $file (@raw_files) {
	if ($file =~ /\*/) {
	    $file =~ s/\$PREFIX/$PREFIX/;
	    $file = "$PREFIX/$file" unless $file =~ /^\//;
	    push(@files,glob($file));
	}
	else {
	    push(@files,$file);
	}
    }
    return @files;
}

foreach my $group_name (keys %{$support_config{group}}) {
    my $group = $support_config{group}->{$group_name};
    add_group($group_name,description => $group->{description});
    foreach my $item_name (keys %{$group->{item}}) {
	my $item = $group->{item}->{$item_name};
	next unless $item and ref($item) eq 'HASH';
	if ($item->{'log'}) {
	    # it's a log file so summarize and attach
	    my @files = _expand_files($item->{'log'});
	    if (@files > 1) {
		foreach my $file (@files) {
		    next unless (-e "$PREFIX/$file" or -e $file);
		    # need a new copy of item for each file
		    my %new_item = %$item;
		    $new_item{text} = sub {
                        my $type = shift;
                        if ($type and $type eq 'display') {
                            get_log($file,$support_config{max_log_lines_display});
                        }
                        else {
                            get_log_compressed($file,$support_config{max_log_lines});
                            }
                        };
		    $new_item{log} = "$file.gz";
		    $new_item{name} = "$file.gz";
		    add($group_name,$file => \%new_item);
	        }
		$item = undef;
	    }
	    elsif (-e "$PREFIX/$files[0]" or -e $files[0]) {
                $item->{text} = sub {
                        my $type = shift;
                        if ($type and $type eq 'display') {
                            get_log($files[0],$support_config{max_log_lines_display});
                        }
                        else {
                            get_log_compressed($files[0],$support_config{max_log_lines});
                        }
                    };
                $item->{file} = "$files[0].gz";
                $item->{name} = "$files[0].gz";
	    }
	}
	elsif ($item->{'file'}) {
	    my @files = _expand_files($item->{'file'});
            my $mask = $item->{mask} if exists($item->{mask});
            my $sub;
            if ($mask) {
                
               $sub = sub {  
                            my $line = shift;
                            eval("\$line =~ $mask");
                            return "Error executing masking: $@" if $@;
                            return $line;
                          };

            }
            
	    if (@files > 1) {
		foreach my $file (@files) {
		    # need a new copy of item for each file
		    my %new_item = %$item;
		    $new_item{text} = sub { get_file($file,$sub) };
		    $new_item{name} = $file;
		    $new_item{file} = $file;
		    add($group_name,"$file" => \%new_item);
	        }
	        $item = undef;
	    }
	    else {
		$item->{text} = sub { get_file($files[0],$sub) };
	    }
	}
	elsif ($item->{'program'}) {
	    $item->{text} = sub { get_program_output($item->{'program'}) }; 
	}
	elsif ($item->{'eval'}) {
	    $item->{text} =  sub {  my $result = eval($item->{'eval'}); $@ ? $@ : $result;}; 
	    #$item->{text} = sub { { my $result = eval($item->{'eval'}) }; $@ ? $@ : $result;}; 
	}
        elsif ($item->{'resource'}) {
            my $mask = $item->{mask} if exists($item->{mask});
            my $sub;
            if ($mask) {
                
               $sub = sub {  
                            my $line = shift;
                            eval("\$line =~ " . $mask);
                            return "Error executing masking: $@" if $@;
                            return $line;
                          };

            }
 
           $item->{text} = sub { get_resource_data($item->{'resource'},$sub) } ;
        }
	add($group_name,$item_name => $item) if $item;
    }
}



1;
