package PureMessage::Support::Data::system_information;
use strict;
use PureMessage::Config;
use PureMessage::_sys;
use PureMessage::Support::Data::Base qw(add add_group get_log get_file get_program_output);
use PureMessage::Support::Config qw( %support_config);
use ActiveState::Unix::Crontab qw(cron_list);
use ActiveState::Unix::ProcInfo qw(proc_info);
use ActiveState::Unix::DiskFree qw(df);
use HTTP::Date qw/time2isoz/;

my %info = ( system_info   => { description => 'System Information',text => \&sys_info, name=>"System Info" },
             crontab       => {description => "Crontab", text => \&gen_cron_list, name=>"crontab" },
             get_proc_info => {description => "Process Info", text => \&get_proc_info, name=>"proc_info" },
           );

add('system_information',%info);

sub gen_cron_list {
    my @lines = cron_list();
    return join("",@lines);
}

sub get_proc_info {
    my @keys;
    my $text;
    for my $p (proc_info()) {
	my $pid = delete($p->{pid});
	my $args = delete($p->{args});
	@keys = sort(keys %$p) unless @keys;
	$text .= "$pid:$args\n";
	$text .= "\t\t$_: $p->{$_} " foreach @keys;
	$text .= "\n";
    }
    $text;
}

sub sys_info {
    my $cpu_count = PureMessage::_sys::available_cpus();
    my $memory = PureMessage::_sys::get_phys_mem();
    my $perl_info = get_program_output("$^X -V");
    my $arch = $^O;

    if ($arch eq 'solaris') {
        my $isainfo = `isainfo 2>/dev/null`;
        $arch .= '64' unless ($? or $isainfo !~ /sparcv9/);
    }

    my @disk_info;
    eval {
	@disk_info = df();
    };

    my $uname = `uname -a`;
    if ($uname) {
	# freebsd, and maybe others, have multiple lines
	chomp $uname;
	$uname =~ s/\n/ /sg;
	$arch .= " '$uname'";
    }

    # OS info for all platforms
    my $text = <<EOF;
System Information
------------------
Memory    : $memory
# of CPUs : $cpu_count
OS        : $arch
EOF

    # additional OS info for some platforms
    if ($arch =~ /linux/i) {
        my ($proc_version, $etc_issue);
        if ($proc_version = get_file('/proc/version')) {
            chomp $proc_version;
            $text .= "/proc/version : $proc_version\n";
        }
        if ($etc_issue = get_file('/etc/issue')) {
            chomp $etc_issue;
            $text .= "/etc/issue    : $etc_issue\n";
        }
    }

    # disk, Perl info (all platforms)
    $text .= <<EOF;
Disk Information
(name, size, used, free, used_p, mountpoint)
----------------
EOF
foreach (@disk_info) {
    $_->{used_p} ||= 'n/a';
    $text .= join(', ', @$_{qw(filesystem size used free used_p mountpoint)});
    $text .= "\n";
}

    $text .= <<EOF;

Perl Version Info
-----------------
$perl_info
EOF

    return $text;
}

1;
